package com.sun.corba.se.impl.ior;

import com.sun.corba.se.spi.protocol.CorbaServerRequestDispatcher;
import org.omg.CORBA_2_3.portable.InputStream;
import org.omg.CORBA_2_3.portable.OutputStream;
import com.sun.corba.se.spi.ior.ObjectId;
import java.util.Iterator;
import com.sun.corba.se.spi.orb.ORBVersion;
import com.sun.corba.se.impl.logging.IORSystemException;
import com.sun.corba.se.spi.orb.ORB;
import com.sun.corba.se.spi.ior.ObjectAdapterId;
import com.sun.corba.se.spi.ior.ObjectKeyTemplate;

public abstract class ObjectKeyTemplateBase implements ObjectKeyTemplate
{
    public static final String JIDL_ORB_ID = "";
    private static final String[] JIDL_OAID_STRINGS;
    public static final ObjectAdapterId JIDL_OAID;
    private ORB orb;
    protected IORSystemException wrapper;
    private ORBVersion version;
    private int magic;
    private int scid;
    private int serverid;
    private String orbid;
    private ObjectAdapterId oaid;
    private byte[] adapterId;
    
    public byte[] getAdapterId() {
        return (byte[])(byte[])this.adapterId.clone();
    }
    
    private byte[] computeAdapterId() {
        ByteBuffer byteBuffer = new ByteBuffer();
        byteBuffer.append(this.getServerId());
        byteBuffer.append(this.orbid);
        byteBuffer.append(this.oaid.getNumLevels());
        Iterator<Object> iterator = this.oaid.iterator();
        while (iterator.hasNext()) {
            byteBuffer.append((String)(String)iterator.next());
        }
        byteBuffer.trimToSize();
        return byteBuffer.toArray();
    }
    
    public ObjectKeyTemplateBase(ORB orb, int magic, int scid, int serverid, String orbid, ObjectAdapterId oaid) {
        this.orb = orb;
        this.wrapper = IORSystemException.get(orb, "oa.ior");
        this.magic = magic;
        this.scid = scid;
        this.serverid = serverid;
        this.orbid = orbid;
        this.oaid = oaid;
        this.adapterId = this.computeAdapterId();
    }
    
    @Override
    public boolean equals(Object o) {
        if (!(o instanceof ObjectKeyTemplateBase)) {
            return false;
        }
        ObjectKeyTemplateBase objectKeyTemplateBase = (ObjectKeyTemplateBase)o;
        return this.magic == objectKeyTemplateBase.magic && this.scid == objectKeyTemplateBase.scid && this.serverid == objectKeyTemplateBase.serverid && this.version.equals(objectKeyTemplateBase.version) && this.orbid.equals(objectKeyTemplateBase.orbid) && this.oaid.equals(objectKeyTemplateBase.oaid);
    }
    
    @Override
    public int hashCode() {
        return 37 * (37 * (37 * (37 * (37 * (37 * 17 + this.magic) + this.scid) + this.serverid) + this.version.hashCode()) + this.orbid.hashCode()) + this.oaid.hashCode();
    }
    
    public int getSubcontractId() {
        return this.scid;
    }
    
    public int getServerId() {
        return this.serverid;
    }
    
    public String getORBId() {
        return this.orbid;
    }
    
    public ObjectAdapterId getObjectAdapterId() {
        return this.oaid;
    }
    
    public void write(ObjectId objectId, OutputStream outputStream) {
        this.writeTemplate(outputStream);
        objectId.write(outputStream);
    }
    
    public void write(OutputStream outputStream) {
        this.writeTemplate(outputStream);
    }
    
    protected abstract void writeTemplate(OutputStream p0);
    
    protected int getMagic() {
        return this.magic;
    }
    
    public void setORBVersion(ORBVersion version) {
        this.version = version;
    }
    
    public ORBVersion getORBVersion() {
        return this.version;
    }
    
    protected byte[] readObjectKey(InputStream inputStream) {
        int read_long = inputStream.read_long();
        byte[] array = new byte[read_long];
        inputStream.read_octet_array(array, 0, read_long);
        return array;
    }
    
    public CorbaServerRequestDispatcher getServerRequestDispatcher(ORB orb, ObjectId objectId) {
        return orb.getRequestDispatcherRegistry().getServerRequestDispatcher(this.scid);
    }
    
    static {
        JIDL_OAID_STRINGS = new String[] { "TransientObjectAdapter" };
        JIDL_OAID = new ObjectAdapterIdArray(ObjectKeyTemplateBase.JIDL_OAID_STRINGS);
    }
}
